@@ -5,11 +5,11 @@ from pathlib import Path
 
 from jsonpickle import decode
 
-from .envelope import Envelope
+from .envelope import Envelope, __doc__ as doc
 from .utils import Attachment
 
 
-class SmartFormatter(argparse.HelpFormatter):
+class SmartFormatter(argparse.RawDescriptionHelpFormatter):
 
     def _split_lines(self, text, width):
         if text.startswith('R|'):
@@ -56,11 +56,12 @@ def _get_envelope(instance: Envelope, args):
 
 
 def main():
-    parser = argparse.ArgumentParser(description=__doc__, formatter_class=SmartFormatter)
+    parser = argparse.ArgumentParser(description=doc, formatter_class=SmartFormatter)
     group_io = parser.add_argument_group("Input/Output")
     group_io.add_argument('--message', help='Plain text message. Empty to read.',
                           metavar="TEXT", nargs="?", action=BlankTrue)
-    group_io.add_argument('--input', help='Path to message file. (Alternative to `message` parameter.)', metavar="FILE")
+    group_io.add_argument('--input', help='Path to message file. (Alternative to the `message` parameter.)',
+                          metavar="FILE")
     group_io.add_argument('--output',
                           help='Path to file to be written to (else the contents is returned if ciphering or True if sending).',
                           metavar="FILE")
@@ -77,42 +78,46 @@ def main():
                             action=BlankTrue, metavar="FINGERPRINT|CONTENTS")
     group_ciph.add_argument('--cert', help='S/MIME: Certificate contents if not included in the key.',
                             action=BlankTrue, metavar="CONTENTS")
-    group_ciph.add_argument('--passphrase', help='If signing key needs passphrase.')
+    group_ciph.add_argument('--passphrase', help='Passphrase to the signing key if needed.')
     group_ciph.add_argument('--sign-path',
-                            help='Filename with the sender\'s private key. (Alternative to `sign` parameter.)',
+                            help='Filename with the from\'s private key. (Alternative to the `sign` parameter.)',
                             metavar="KEY-PATH")
     group_ciph.add_argument('--cert-path', help='S/MIME: Filename with the sender\'s S/MIME private cert'
-                                                ' if cert not included in the key. (Alternative to `cert` parameter.)',
+                                                ' if cert not included in the key.'
+                                                ' (Alternative to the `cert` parameter.)',
                             metavar="CERT-PATH")
 
     group_ciph.add_argument('--encrypt', help='R|* GPG:'
                                               "\n  * Blank for user default key"
+                                              "\n  * \"auto\" for turning on encrypting if there is a matching key for every recipient"
                                               "\n  * key ID/fingerprint"
                                               "\n  * Any attainable contents with the key to be signed with"
                                               " (will be imported into keyring)"
-                                              "\n  * \"auto\" for turning on encrypting if there is a matching key for every recipient"
+                                              "\n  * list of the identities specified by key ID / fingerprint / e-mail address / raw key data"
                                               "\n* S/MIME any attainable contents with certificate to be encrypted with or their list",
                             nargs="*", action=BlankTrue, metavar="GPG-KEY/SMIME-CERTIFICATE-CONTENTS")
-    group_ciph.add_argument('--encrypt-path', help='Filename(s) with the recipient\'s public key.'
-                                                   ' (Alternative to `encrypt` parameter.)',
+    group_ciph.add_argument('--encrypt-path', help='Filename(s) with the recipient\'s public key(s).'
+                                                   ' (Alternative to the `encrypt` parameter.)',
                             nargs="*", metavar="PATH")
-    group_ciph.add_argument('--attach-key', help="Appending public key to the attachments when sending.",
+    group_ciph.add_argument('--attach-key', help="Append GPG public key as an attachment when sending.",
                             action="store_true")
 
     group_recip = parser.add_argument_group("Recipients")
     group_recip.add_argument('-t', '--to', help="E-mail – needed to choose their key if encrypting", metavar="E-MAIL",
-                            nargs="*", action=BlankTrue)
+                             nargs="*", action=BlankTrue)
     group_recip.add_argument('--cc', help="E-mail or list", metavar="E-MAIL", nargs="*", action=BlankTrue)
     group_recip.add_argument('--bcc', help="E-mail or list", metavar="E-MAIL", nargs="*", action=BlankTrue)
     group_recip.add_argument('--reply-to',
-                            help="Header that states e-mail to be replied to. The field is not encrypted.",
-                            metavar="E-MAIL", nargs="?", action=BlankTrue)
+                             help="Header that states e-mail to be replied to. The field is not encrypted.",
+                             metavar="E-MAIL", nargs="?", action=BlankTrue)
     group_recip.add_argument('-f', '--from', help="E-mail – needed to choose our key if encrypting", metavar="E-MAIL",
-                            nargs="?", action=BlankTrue)
+                             nargs="?", action=BlankTrue)
     group_recip.add_argument('--sender', help="Alias for --from if not set."
                                               " Otherwise appends the \"Sender\" header.", metavar="E-MAIL")
-    group_recip.add_argument('--no-sender', action="store_true",
-                            help="We explicitly say we do not want to decipher later if encrypting.")
+    group_recip.add_argument('--no-from', action="store_true",
+                             help="We explicitly say we do not want to decipher later if encrypting.")
+    group_recip.add_argument('--from-addr', help="SMTP envelope MAIL FROM address", metavar="E-MAIL",
+                             nargs="?", action=BlankTrue)
 
     group_send = parser.add_argument_group("Sending")
     group_send.add_argument('-s', '--subject', help="E-mail subject", nargs="?", action=BlankTrue)
@@ -130,9 +135,11 @@ def main():
     group_send.add_argument('--mime', help="Set contents mime subtype: 'html' (default) or 'plain' for plain text",
                             metavar="SUBTYPE")
     group_send.add_argument('--smtp',
-                            help="SMTP server. List `host, [port, [username, password, [security]]]` or dict.\n"
+                            help="SMTP server. List `host, [port, [username, password,"
+                                 " [security, [timeout, [attempts, [delay]]]]]]` or dict.\n"
                                  "Ex: '--smtp {\"host\": \"localhost\", \"port\": 25}'."
-                                 " Security may be explicitly set to 'starttls', 'tls' or automatically determined by port.",
+                                 " Security may be explicitly set to 'starttls', 'tls'"
+                                 " or automatically determined by port.",
                             nargs="*", action=BlankTrue, metavar=("HOST", "PORT"))
     group_send.add_argument('--send', help="Send e-mail. Blank to send now.", nargs="?", action=BlankTrue)
 
@@ -177,19 +184,24 @@ def main():
     del args["input"]
 
     # we explicitly say we do not want to decipher later if encrypting
-    if args["no_sender"]:
+    if args["no_from"]:
         args["from_"] = False
-    del args["no_sender"]
+    else:
+        args["from_"] = args["from"]
+    del args["from"]
+    del args["no_from"]
 
     # user is saying that encryption key has been already imported
     enc = args["encrypt"]
     if enc and enc is not True:
-        if enc.lower() in ["1", "true", "yes"]:
+        if type(args["encrypt"]) is list:
+            pass
+        elif enc.lower() in ["1", "true", "yes"]:
             args["encrypt"] = True
         elif enc.lower() in ["0", "false", "no"]:
             args["encrypt"] = False
 
-    # user specified encrypt key in a path. And did not disabled encryption
+    # user specified encrypt key in a path. And did not disable encryption
     if args["encrypt_path"] and args["encrypt"] is not False:
         if args["encrypt"] not in [True, None]:  # user has specified both path and the key
             raise RuntimeError("Cannot define both encrypt key data and encrypt key path.")
@@ -199,7 +211,7 @@ def main():
             args["encrypt"] = Path(args["encrypt_path"])
     del args["encrypt_path"]
 
-    # user specified sign key in a path. And did not disabled signing
+    # user specified sign key in a path. And did not disable signing
     if args["sign_path"] and args["sign"] is not False:
         if args["sign"] not in [True, None]:  # user has specified both path and the key
             raise RuntimeError("Cannot define both sign key data and sign key path.")
@@ -237,9 +249,6 @@ def main():
     args["headers"] = args["header"]
     del args["header"]
 
-    args["from_"] = args["from"]
-    del args["from"]
-
     check = args.pop("check")
     preview = args.pop("preview")
     read_attachments = args.pop("read_attachments")
