@@ -1,15 +1,19 @@
-import io
 import logging
-import smtplib
 import ssl
 from collections import defaultdict
 from email.utils import getaddresses, parseaddr
+from io import TextIOBase, BufferedIOBase
+from os import environ
 from pathlib import Path
-from socket import gaierror, timeout
-from typing import Union
+from smtplib import SMTP, SMTP_SSL, SMTPAuthenticationError, SMTPException, SMTPSenderRefused
+from socket import gaierror, timeout as timeout_exc
+from time import sleep
+from typing import Optional, Tuple, Union, Dict, Type, Iterable
 
 import magic
-from validate_email import validate_email  # package py3-validate-email
+
+environ['PY3VE_IGNORE_UPDATER'] = '1'
+from validate_email import validate_email  # noqa, #17 E402, the package name is py3-validate-email
 
 logger = logging.getLogger(__name__)
 
@@ -189,8 +193,10 @@ class Attachment:
             logger.error(f"Could not fetch file {contents.absolute()}")
             raise
         if not mimetype:
-            m = magic.Magic(mime=True)
-            mimetype = m.from_file(str(contents)) if isinstance(contents, Path) else m.from_buffer(contents)
+            if isinstance(contents, Path):
+                mimetype = get_mimetype(path=contents)
+            else:
+                mimetype = get_mimetype(data=data)            
 
         self.data: bytes = data
         self.mimetype = mimetype
@@ -251,10 +257,10 @@ class AutoSubmittedHeader:
 
 
 class _Message:
-    auto: bytes = None
-    plain: bytes = None
-    html: bytes = None
-    boundary: str = None  # you may specify e-mail boundary used when multiple alternatives present
+    auto: Optional[bytes] = None
+    plain: Optional[bytes] = None
+    html: Optional[bytes] = None
+    boundary: Optional[str] = None  # you may specify e-mail boundary used when multiple alternatives present
 
     def _decode(self, val, mime_alternative):
         try:
@@ -265,7 +271,7 @@ class _Message:
             val = str(val)
         return val
 
-    def get(self, type_: Union[bytes, str] = bytes) -> (Union[bytes, str, None], Union[bytes, str, None]):
+    def get(self, type_: Union[Type[bytes], Type[str]] = bytes) -> Tuple[Union[bytes, str, None], Union[bytes, str, None]]:
         """
         :param type_: Set to str if we should return str instead of bytes.
          Note it raises a warning if bytes were not in Unicode.
@@ -277,6 +283,19 @@ class _Message:
             raise ValueError("Specified all of message alternative=plain, alternative=html and alternative=auto,"
                              " choose only two.")
 
+        # XX As of Python3.10 replace with longer but more readable:
+        # plain, html = self.plain, self.html
+        # if self.auto:
+            # match bool(plain), bool(html):
+            #     case False, _:
+            #         plain = auto
+            #     case _, False:
+            #         html = auto
+            #     case _:
+        #             raise ValueError("Specified all of message alternative=plain, alternative=html and alternative=auto,"
+        #                      " choose only two.")
+
+
         if type_ is str:
             if plain:
                 plain = self._decode(plain, "plain")
@@ -295,13 +314,17 @@ class _Message:
             return text or html
 
 
-class SMTP:
+class SMTPHandler:
     # cache of different smtp connections.
     # Usecase: user passes smtp server info in dict in a loop but we do want it connects just once
-    _instances = {}
+    _instances: Dict[str, SMTP] = {}
 
-    def __init__(self, host="localhost", port=25, user=None, password=None, security=None):
-        if isinstance(host, smtplib.SMTP):
+    def __init__(self, host="localhost", port=25, user=None, password=None, security=None, timeout=3, attempts=3,
+                 delay=3):
+        self.attempts = attempts
+        self.delay = delay  # If sending timeouts, delay N seconds before another attempt.
+
+        if isinstance(host, SMTP):
             self.instance = host
         else:
             self.instance = None
@@ -310,6 +333,7 @@ class SMTP:
             self.user = user
             self.password = password
             self.security = security
+            self.timeout = timeout
         d = locals()
         del d["self"]
         self.key = repr(d)
@@ -323,18 +347,18 @@ class SMTP:
 
             context = ssl.create_default_context()
             if self.security == "tls":
-                smtp = smtplib.SMTP_SSL(self.host, self.port, timeout=1, context=context)
+                smtp = SMTP_SSL(self.host, self.port, timeout=self.timeout, context=context)
             else:
-                smtp = smtplib.SMTP(self.host, self.port, timeout=1)
+                smtp = SMTP(self.host, self.port, timeout=self.timeout)
                 if self.security == "starttls":
                     smtp.starttls(context=context)
             if self.user:
                 try:
                     smtp.login(self.user, self.password)
-                except smtplib.SMTPAuthenticationError as e:
+                except SMTPAuthenticationError as e:
                     logger.error(f"SMTP authentication failed: {self.key}.\n{e}")
                     return False
-        except smtplib.SMTPException as e:
+        except SMTPException as e:
             logger.error(f"SMTP connection failed: {self.key}.\n{e}")
             return False
         except (gaierror, ConnectionError):
@@ -342,8 +366,8 @@ class SMTP:
             return False
         return smtp
 
-    def send_message(self, email, to_addrs):
-        for attempt in range(1, 3):  # an attempt to reconnect possible
+    def send_message(self, email, from_addr, to_addrs):
+        for attempt in range(self.attempts):  # an attempt to reconnect possible
             try:
                 if self.key not in self._instances:
                     self._instances[self.key] = self.connect()
@@ -352,35 +376,71 @@ class SMTP:
                     return False
 
                 # recipients cannot be taken from headers when encrypting, we have to re-list them again
-                return smtp.send_message(email, to_addrs=to_addrs)
-
-            except (timeout, smtplib.SMTPSenderRefused) as e:  # timeouts
-                if attempt == 2:
-                    logger.warning(f"SMTP sender refused, unable to reconnect.\n{e}")
+                return smtp.send_message(email, from_addr=from_addr, to_addrs=to_addrs)
+            except (timeout_exc, SMTPException) as e:
+                del self._instances[self.key]  # this connection is gone, reconnect next time
+                if isinstance(e, SMTPSenderRefused):
+                    logger.warning(f"SMTP sender refused, unable to reconnect. {e}")
                     return False
-                del self._instances[self.key]  # this connection is gone possibly due to a timeout, reconnect
-                continue
-            except smtplib.SMTPException as e:
-                logger.error(f"SMTP sending failed.\n{e}")
-                return False
+                elif isinstance(e, timeout_exc):
+                    if self.delay:
+                        sleep(self.delay)
+                    continue
+                elif isinstance(e, SMTPException):
+                    if attempt + 1 < self.attempts:
+                        logger.info(f"{type(e).__name__}, attempt {attempt + 1}. {e}")
+                        if self.delay:
+                            sleep(self.delay)
+                        continue
+                    else:
+                        logger.warning(f"{type(e).__name__}: sending failed. {e}")
+                        return False
+
+    def quit(self):
+        if self.key in self._instances:
+            self._instances[self.key].quit()
+
+    @classmethod
+    def quit_all(cls):
+        [c.quit() for c in cls._instances.values()]
 
 
-def is_gpg_fingerprint(key):
-    """ Check if we have key fingerprint in the variable or the key contents itself """
-    return isinstance(key, str) and len(key) * 4 < 512  # 512 is the smallest possible GPG key
+def is_gpg_importable_key(key):
+    """ Check if the variable contains the key contents itself
+     (it may contain a fingerprint or an e-mail address too) """
+    return len(key) >= 512  # 512 is the smallest possible GPG key
 
 
-def assure_list(l):
-    """ Accepts object and returns list, if object is not list, it's appended to a list. If None, returns empty list.
+def get_mimetype(data:bytes=None, path:Path=None):
+    """ We support both python-magic and file-magic library, any of them can be on the system. #25
+        Their funcionality is the same, their API differs.
+    """
+    # XX change to match statement as of Python3.10
+    if hasattr(magic.Magic, "from_file"):  # this is python-magic
+        if data:
+            return magic.Magic(mime=True).from_buffer(data)
+        if path:
+            return magic.Magic(mime=True).from_file(str(path))
+    else:  # this is file-magic
+        if data:
+            return magic.detect_from_content(data).mime_type
+        if path:
+            return magic.detect_from_filename(str(path)).mime_type
+
+def assure_list(v):
+    """ Accepts object and returns list.
+    If object is tuple, generator, set, frozenset, it's converted to a list.
+    If object is not a list, it's appended to a list.
+    If None, returns an empty list.
         "test" → ["test"]
-        (5,1) → [(5,1)]
+        (5,1) → [5,1]
         ["test", "foo"] → ["test", "foo"]
     """
-    if l is None:
+    if v is None:
         return []
-    elif type(l) is not list:
-        return [l]
-    return l
+    if isinstance(v, Iterable) and not isinstance(v, (TextIOBase, BufferedIOBase, str, bytes)):
+        return list(v)
+    return [v]
 
 
 def assure_fetched(message, retyped=None):
@@ -398,13 +458,13 @@ def assure_fetched(message, retyped=None):
         return None
     elif isinstance(message, Path):
         message = message.read_bytes()
-    elif isinstance(message, (io.TextIOBase, io.BufferedIOBase)):
+    elif isinstance(message, (TextIOBase, BufferedIOBase)):
         message = message.read()
-    elif type(message) not in [str, bytes, bool]:
+    elif not isinstance(message, (str, bytes, bool)):
         raise ValueError(f"Expected str, bytes, stream or pathlib.Path: {message}")
 
-    if retyped is bytes and type(message) is str:
+    if retyped is bytes and isinstance(message, str):
         message = message.encode("utf-8")
-    elif retyped is str and type(message) is bytes:
+    elif retyped is str and isinstance(message, bytes):
         message = message.decode("utf-8")
     return message
